{% extends "!layout.html" %}
{% block footer %}
{{ super() }}
<script type="text/javascript">
//<![CDATA[
    var gaJsHost = (("https:" == document.location.protocol) ? "https://ssl." : "http://www.");
    document.write(unescape("%3Cscript src='" + gaJsHost + "google-analytics.com/ga.js' type='text/javascript'%3E%3C/script%3E"));
    //]]>
    </script><script type="text/javascript">
//<![CDATA[
    try{
        var pageTracker = _gat._getTracker("UA-1048253-4");
        pageTracker._trackPageview();
    } catch(err) {}
    //]]>
    </script>
{% endblock %}
