<h3>Download</h3>
<p>Current version: <b>{{ version }}</b></p>

<p>Download StarCluster from the
<a href="http://github.com/jtriley/StarCluster">github project page</a> (but first take a look at the
<a href="{{ pathto('installation') }}">install</a> page).</p>

<p>For details on what's new, see the detailed <a href="{{
pathto('changelog') }}">version history</a>.</p>

<h3>Support</h3>

<p>Requests for help should be directed to the
<a href="http://star.mit.edu/cluster/mailinglist.html">StarCluster mailing list</a>.</p>

<p>You can file bugs, patches and feature requests on the
<a href="http://github.com/jtriley/StarCluster/issues">StarCluster bug tracker</a>,
but it is a good idea to also drop us a note on the
<a href="http://star.mit.edu/cluster/mailinglist.html">StarCluster
Mailing List</a>.

<h3>License</h3>

<p>The StarCluster module is released under the LGPL license. See the
<a href="{{ pathto('license') }}">license</a> page for details.</p>
