{% extends "basic/layout.html" %}

{%- block extrahead %}
<link rel="stylesheet" href="{{ pathto('_static/nobile.css', 1) }}" type="text/css" media="screen" charset="utf-8" />
<link rel="stylesheet" href="{{ pathto('_static/neuton.css', 1) }}" type="text/css" media="screen" charset="utf-8" />
<!--[if lte IE 6]>
<link rel="stylesheet" href="{{ pathto('_static/ie6.css', 1) }}" type="text/css" media="screen" charset="utf-8" />
<![endif]-->
{%- if theme_favicon %}
<link rel="shortcut icon" href="{{ pathto('_static/'+theme_favicon, 1) }}"/>
{%- endif %}
{% endblock %}

{% block header %}
<div class="header-small">
	{%- if theme_logo %}
	{% set img, ext = theme_logo.split('.', -1) %}
	<div class="logo-small">
		<a href="{{ pathto(master_doc) }}">
      		<img class="logo" src="{{ pathto('_static/%s-small.%s' % (img, ext), 1)}}" alt="Logo"/>
		</a>
  	</div>
  	{%- endif %}
</div>
{% endblock %}

{%- macro relbar() %}
<div class="related">
	<h3>{{ _('Navigation') }}</h3>
	<ul>
		{%- for rellink in rellinks %}
		<li class="right" {% if loop.first %}style="margin-right: 10px"{% endif %}>
			<a href="{{ pathto(rellink[0]) }}" title="{{ rellink[1]|striptags|e }}"
			{{ accesskey(rellink[2]) }}>{{ rellink[3] }}</a>
			{%- if not loop.first %}{{ reldelim2 }}{% endif %}
		</li>
		{%- endfor %}
    	{%- block rootrellink %}
    	<li><a href="{{ pathto(master_doc) }}">{{ theme_short_title|e }}</a>{{ reldelim1 }}</li>
    	{%- endblock %}
    	{%- for parent in parents %}
          <li><a href="{{ parent.link|e }}" {% if loop.last %}{{ accesskey("U") }}{% endif %}>{{ parent.title }}</a>{{ reldelim1 }}</li>
        {%- endfor %}
        {%- block relbaritems %} {% endblock %}
  	</ul>
</div>
{%- endmacro %}

{%- block sidebarlogo %}{%- endblock %}
{%- block sidebarsourcelink %}{%- endblock %}

{% block footer %}
{{ super() }}
{% if gatracker %}
</script><script type="text/javascript">
//<![CDATA[
    var gaJsHost = (("https:" == document.location.protocol) ? "https://ssl." : "http://www.");
    document.write(unescape("%3Cscript src='" + gaJsHost + "google-analytics.com/ga.js' type='text/javascript'%3E%3C/script%3E"));
    //]]>
    </script><script type="text/javascript">
//<![CDATA[
    try{
        var pageTracker = _gat._getTracker("{{ gatracker }}");
        pageTracker._trackPageview();
    } catch(err) {}
    //]]>
</script>
{% endif %}
{% endblock %}
