{% extends "layout.html" %}
{% set title = 'Home' %}

{%- macro relbar() %}
<div class="related">
	<h3>{{ _('Navigation') }}</h3>
	<ul>
		{%- for rellink in rellinks %}
		<li class="right" {% if loop.first %}style="margin-right: 10px"{% endif %}>
			<a href="{{ pathto(rellink[0]) }}" title="{{ rellink[1]|striptags|e }}"
			{{ accesskey(rellink[2]) }}>{{ rellink[3] }}</a>
			{%- if not loop.first %}{{ reldelim2 }}{% endif %}
		</li>
		{%- endfor %}
    	{%- block rootrellink %}
    	<li><a href="{{ pathto(master_doc) }}">{{ theme_short_title|e }}</a>{{ reldelim2 }}</li>
        <li><a href="{{ pathto('installation') }}">Install</a>{{ reldelim2 }}</li>
        <li><a href="{{ pathto('quickstart') }}">Quick-Start</a>{{ reldelim2 }}</li>
        <li><a href="{{ pathto('manual/configuration') }}">Config</a>{{ reldelim2 }}</li>
        <li><a href="{{ pathto('manual/index') }}">Manual</a>{{ reldelim2 }}</li>
        <li><a href="{{ pathto('contribute') }}">Contribute</a>{{ reldelim2 }}</li>
        <li><a href="{{ pathto('support') }}">Support</a></li>
    	{%- endblock %}
    	{%- for parent in parents %}
          <li><a href="{{ parent.link|e }}" {% if loop.last %}{{ accesskey("U") }}{% endif %}>{{ parent.title }}</a>{{ reldelim1 }}</li>
        {%- endfor %}
        {%- block relbaritems %} {% endblock %}
  	</ul>
</div>
{%- endmacro %}

{% block body %}

  <h1>{{ project }} documentation (v. {{release}})</h1>

  <p>
  StarCluster is a utility for creating and managing distributed computing
  clusters hosted on <a href="http://aws.amazon.com/ec2" title="Amazon EC2">
  Amazon's Elastic Compute Cloud (EC2)</a>. StarCluster utilizes Amazon's EC2
  web service to create and destroy clusters of Linux virtual machines on
  demand.
  </p>

  <p>
  StarCluster aims to be an easy-to-use and extensible cluster computing
  toolkit for the cloud. The sections below are divided into user and developer
  documentation.
  </p>

  <h2>User Documentation</h2>
   <table class="contentstable" align="center">
      <tr valign="top"><td width="50%">
      <p class="biglink">
         <a class="biglink" href="{{ pathto('overview') }}">
            Overview</a><br/>
         <span class="linkdescr">
            Introduction to {{project}}.
         </span></p>

      <p class="biglink">
         <a class="biglink" href="{{ pathto('quickstart') }}">
            Quick-Start Tutorial</a><br/>
         <span class="linkdescr">
            A Quick-Start tutorial for new users.
         </span></p>

     <p class="biglink">
         <a class="biglink" href="{{ pathto('installation') }}">
            Install instructions</a><br/>
         <span class="linkdescr">
            Detailed installation instructions
         </span></p>


      <p class="biglink">
         <a class="biglink" href="{{ pathto('support') }}">
            Support</a><br/>
         <span class="linkdescr">
            Reporting bugs, issues, questions, feature requests, etc.
         </span></p>

   </td><td width="50%">

     <p class="biglink">
         <a class="biglink" href="{{ pathto('manual/index') }}">
            User Manual</a><br/>
         <span class="linkdescr">
            Complete user manual for StarCluster
         </span></p>

     <p class="biglink">
         <a class="biglink" href="{{ pathto('guides/index') }}">
            Guides</a><br/>
         <span class="linkdescr">
            Task-oriented guides
         </span></p>

     <p class="biglink">
         <a class="biglink" href="{{ pathto('plugins/index') }}">
            Plugins</a><br/>
         <span class="linkdescr">
            Plugin-specific documentation
         </span></p>

     <p class="biglink">
         <a class="biglink" href="{{ pathto('faq') }}">
            FAQ</a><br/>
         <span class="linkdescr">
            Frequently Asked Questions
         </span></p>
   </td></tr></table>

  <h2>StarCluster Development</h2>
   <table class="contentstable" align="center">
      <tr valign="top"><td width="50%">
      <p class="biglink">
        <a class="biglink" href="{{ pathto('contribute') }}">
            Contribute</a><br/>
        <span class="linkdescr">
            Taking part in StarCluster's development.
        </span></p>

     <p class="biglink">
         <a class="biglink" href="{{ pathto('hacking') }}">
            Hacking</a><br/>
         <span class="linkdescr">
            Design rationales, source organization.
         </span></p>
   </td><td width="50%">
     <p class="biglink">
         <a class="biglink" href="{{ pathto('features') }}">
            Features</a><br/>
         <span class="linkdescr">
            Current and planned features.
         </span></p>

      <p class="biglink">
        <a class="biglink" href="{{ pathto('TODO') }}">
            TODO</a><br/>
        <span class="linkdescr">
            Remaining development tasks.
        </span></p>

   </td></tr></table>

   <h2>Indices and Tables</h2>

   <table class="contentstable" align="center">
      <tr valign="top"><td width="50%">
      <p class="biglink">
         <a class="biglink" href="{{ pathto('contents') }}">
            Table of Contents</a><br/>
         <span class="linkdescr">
            Lists all sections and subsections.
         </span></p>
      <p class="biglink">
         <a class="biglink" href="{{ pathto('search') }}">
            Search Page</a><br/>
         <span class="linkdescr">
            Search this documentation.
         </span></p>
      </td><td width="50%">
      <p class="biglink">
         <a class="biglink" href="{{ pathto('genindex') }}">
            Index</a><br/>
         <span class="linkdescr">
            All functions, classes, terms.
         </span></p>
    </td></tr></table>
{% endblock %}
