/* Generated by Font Squirrel (http://www.fontsquirrel.com) on February 11, 2011 02:21:27 PM America/New_York */



@font-face {
    font-family: 'NobileRegular';
    src: url('nobile-webfont.eot?') format('eot'),
         url('nobile-webfont.woff') format('woff'),
         url('nobile-webfont.ttf') format('truetype'),
         url('nobile-webfont.svg#webfontn5VEQQo3') format('svg');
    font-weight: normal;
    font-style: normal;

}

@font-face {
    font-family: 'NobileItalic';
    src: url('nobile_italic-webfont.eot?') format('eot'),
         url('nobile_italic-webfont.woff') format('woff'),
         url('nobile_italic-webfont.ttf') format('truetype'),
         url('nobile_italic-webfont.svg#webfontg7NPy4Pa') format('svg');
    font-weight: normal;
    font-style: normal;

}

@font-face {
    font-family: 'NobileBold';
    src: url('nobile_bold-webfont.eot?') format('eot'),
         url('nobile_bold-webfont.woff') format('woff'),
         url('nobile_bold-webfont.ttf') format('truetype'),
         url('nobile_bold-webfont.svg#webfontQ4gbM52t') format('svg');
    font-weight: normal;
    font-style: normal;

}

@font-face {
    font-family: 'NobileBoldItalic';
    src: url('nobile_bold_italic-webfont.eot?') format('eot'),
         url('nobile_bold_italic-webfont.woff') format('woff'),
         url('nobile_bold_italic-webfont.ttf') format('truetype'),
         url('nobile_bold_italic-webfont.svg#webfontPHzhAcVd') format('svg');
    font-weight: normal;
    font-style: normal;

}

