/*

	@file         screen.css
	@description  Screen stylesheet
	vim: set noet ts=4 fdm=marker fenc=utf-8:

*/

@import url("./reset.css");

/* @section Basic */

html { font-size: 62.5%; }
html, body { height: 100%; }

body {
	font-family: Helvetica, Arial, sans-serif;
	font-size: 150%;
	line-height: 1.3;
	color: #FFFFFF;
	background: #666666;
	position: relative;
	padding: 0 30px;
}

p,ul,ol,dl,table,pre { margin-bottom: 1em; }
ul { margin-left: 20px; }
a { text-decoration: none; cursor: pointer; color: #ba832c; font-weight: bold; }
a:focus { outline: 1px dotted; }
a:visited {  }
a:hover, a:focus { color: #d3a459; text-decoration: none; }
a *, button * { cursor: pointer; }
hr { display: none; }
small { font-size: 90%; }
input, select, button, textarea, option { font-family: Arial, "Lucida Grande", "Lucida Sans Unicode", Arial, Verdana, sans-serif; font-size: 100%; }
button, label, select, option, input[type=submit] { cursor: pointer; }
.group:after { content: "."; display: block; height: 0; clear: both; visibility: hidden; } .group {display: inline-block;}
/* Hides from IE-mac \*/ * html .group {height: 1%;} .group {display: block;} /* End hide from IE-mac */
sup { font-size: 80%; line-height: 1; vertical-align: super; }
button::-moz-focus-inner { border: 0; padding: 1px; }
span.amp { font-family: Baskerville, "Goudy Old Style", "Palatino", "Book Antiqua", serif; font-weight: normal; font-style: italic; font-size: 1.2em; line-height: 0.8; }
h1,h2,h3,h4,h5,h6 { line-height: 1.1; }

::selection { background: #745626; }
::-moz-selection { background: #745626; }

h1 {
	font-size: 220%;
	margin: 0 0 0.1em;
	font-family: serif;
	font-weight: normal;
	text-shadow: 1px 1px 10px rgba(0,0,0,0.25);
}

h1 a,
h1 a:hover {
	color: #d7af72;
	font-weight: normal;
	text-decoration: none;
}

#subtitle {
	color: #a1804c;
}

#wrap {
	width: 52em;
	/*width: 760px;*/
	max-width: 100%;
	margin: 0 auto;
	padding: 30px 0 0;
}

#header {
	text-align: center;
	padding: 1em 0 2em;
}

pre {
	background: rgba(0,0,0,0.3);
	color: #fff;
	padding: 8px 10px;
	border-radius: 0.4em;
	-moz-border-radius: 0.4em;
	-webkit-border-radius: 0.4em;
	overflow-x: hidden;
}

pre code {
	font-size: 10pt;
}

/* @section Informations */

#informations {
	border-top: 1px solid rgba(0,0,0,0.5);
}

#informations ul {
	margin: 0;
}

#informations .row {
	border-bottom: 1px solid rgba(0,0,0,0.5);
	border-top: 1px solid rgba(255,255,255,0.08);
	padding: 2em 20px 0;
}
 
#informations .row .col-1 {
	width: 49%;
	float: left;
	padding: 0 0 1em;
}

#informations .row .col-2 {
	width: 49%;
	float: right;
	padding: 0 0 1em;
}

.button {
	text-align: center;
}

#informations .button a {
	background: rgba(162,107,20,0.3);
	padding: 8px 10px 6px;
	border-radius: 0.4em;
	-moz-border-radius: 0.4em;
	-webkit-border-radius: 0.4em;
	box-shadow: 0 0 5px rgba(0,0,0,0.4);
	-moz-box-shadow: 0 0 5px rgba(0,0,0,0.4);
	-webkit-box-shadow: 0 0 5px rgba(0,0,0,0.4);
	text-decoration: none;
}

#informations .button a:hover {
	background: rgba(162,107,20,0.25);
}

#informations .button-large {
	padding: 2em 0 1em;
	font-size: 120%;
}

#informations .quote {
	text-align: center;
	color: #816f51;
	padding-bottom: 2em;
}

#informations .quote blockquote {
	/*font-family: Georgia, serif;*/
	font-size: 140%;
	padding: 0 15%;
}

#informations .quote blockquote span {
	font-family: Georgia, serif;
	font-size: 140%;
	line-height: 0.5;
	vertical-align: sub;
}

#informations .quote cite {
	font-style: normal;
}

#informations .quote cite a {
	font-weight: normal;
}
