<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
 <head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
    <title>TethysCluster - Spot History for {{ instance_type }}</title>
    <link href="/css/screen.css" rel="stylesheet" type="text/css">
    <link href="/css/zoom.css" rel="stylesheet" type="text/css">
    <!--[if lte IE 8]><script language="javascript" type="text/javascript" src="/js/excanvas.min.js"></script><![endif]-->
    <script language="javascript" type="text/javascript" src="/js/jquery.js"></script>
    <script language="javascript" type="text/javascript" src="/js/jquery.flot.js"></script>
    <script language="javascript" type="text/javascript" src="/js/jquery.flot.navigate.js"></script>
    <script language="javascript" type="text/javascript" src="/js/shutdown.js"></script>
 </head>
    {% if shutdown %}
    <body onload="javascript: shutdown()">
    {% else %}
    <body>
    {% endif %}
    <div id="wrap">
        <div id="header">
            <h1>TethysCluster - Spot History for {{ instance_type}}</h1>
        </div>
        <center>
        <p><i>Spot price during {{ start }} - {{ end }} (weekends are highlighted)</i></p>
        <p><i>All timestamps are in UTC</i></p>
        <div id="placeholder" style="width:800px;height:400px;"></div>
        <p>
        <div id="row" class="group row">
            <div id="console" class="col-2">
                <pre style="width:405px;height:20px"><code><span style="color: #e3d796;"><b>TethysCluster Server Status: </b></span><span id="console_text">Shutting down server...</span></code></pre>
            </div>
        </div>
        </p>
        </center>
    </div>
    <script language="javascript" type="text/javascript" src="/js/timeseries.js"></script>
    </body>
</html>
